// Functions for SSD1306/SSD1106 OLED Driver
// Version 4.0 by Nigel Mercier, based on code by dariuskrail

#include "SH1106.h"
#include "font5.h"
#include "font6R.h"
#include "functions.h"

// SSD1307:
// StartCol = 0;
// DisplayRAMColSize = 128;
// SH1106:
// StartCol = 2;
// DisplayRAMColSize = 132;

// ** Functions **
// Display Internal functions

void initDisplay(void)
{
  DisplayRST = 0 ;  // reset display (RST is active LOW)
  delayX25ms(1) ;   // minimum 10uS
  DisplayRST = 1 ;  // release reset

  sendCmd(0xAE) ;    // turn off OLED panel                                           [#11]

  sendCmd(0xC8) ;    // set COM scan direction 0-n [POR = C0, C8]                     [#13]
  sendCmd(0xA1) ;    // set SEG/Column left/right re-map [POR = A0, A1]                [#6]
//  sendCmd(0xA4) ;    // Entire Display Force On [POR = A4, A5 force on]                [#7]
  
  clsDisplay() ;     // clear RAM data and zero cursor
  setContrast(128) ; // POR value = 0x80
  sendCmd(0xAF) ;    // turn on OLED panel                                        [#11]
  delayX25ms(4) ;    // at least 100mS after 0xAF command
}

void sendCmd(byte cmdOut)
{ // Write a command to Display
   DisplayD_C = 0 ;     // set A0 low, select Command mode DisplayD_C = 0 for this byte
   writeSD(cmdOut) ;    // Clock out command bits
   DisplayD_C = 1 ;     // set A0 high, select Data mode DisplayD_C = 1 for next ...
}

void sendData(byte dataOut)
{ // Write data to Display, DisplayD_C = 1 set after sendCmd()
  //  DisplayD_C = 1 ;     // set A0 high, select Data mode DisplayD_C = 1
  writeSD(dataOut) ;   // Clock out data bits
}

void writeSD(byte byteOut)
{ // Clocks out data to the SSD1306/SSD1106 controller
  signed short int bitcnt ;
  for (bitcnt = 8 ; bitcnt > 0 ; bitcnt--)
  {
    DisplayCLK = 0 ;           // Set clock low
    DisplaySDI = byteOut.B7 ;  // transfer current bit 7 of byteOut to DisplaySDI pin
    DisplayCLK = 1 ;           // clock data bit on the rising edge of DisplayCLK
    byteOut = byteOut << 1 ;   // logical shift left: get next bit to b7
  }
}

void sendGlyphs(const byte *pRow, byte size)
{ //Send any font row to print character
  byte count ;
  for (count = 0 ; count < size ; count++)  // Send row to print the character
  {
  sendData((pRow[count])) ;
  }
  sendData(0x00);  // 1 pixel horizontal space after character
}

// Display Screen Controls

void setContrast(unsigned char contr)
{ 
  sendCmd(0x81) ;    // set contrast control register                             [#5]
  sendCmd(contr) ;   //
}

void cursorY(unsigned char add)
{ //Set page address for Page Addressing Mode
  sendCmd(0xB0 | add) ;
}

void cursorX(unsigned char add)
{ //Set column adress for Page Addressing Mode
  add = add+sXOffset ;
  sendCmd(0x10 | (add>>4)) ; //  shift high 4
  sendCmd(0x0F & add) ;      // low 4
}

void cursorXY(byte x, byte y)
{ // Position cursor to column, line
  cursorY(y) ; // Y axis = line 0-7
  cursorX(x) ; // X axis = column 0 - (sWidth - 1)
}

void onDisplay(void)
{
  sendCmd(0xAF) ;                     // turn on OLED panel
}

void offDisplay(void)
{
  sendCmd(0xAE) ;                     // turn off OLED panel
}

void clsDisplay(void)
{ // Erase the entire display DDRAM and set cursor home (0,0)
  byte line;
  offDisplay() ;                      // turn off OLED panel
  for (line = 8 ; line > 0 ; line--)  // 8 lines of Display DDRAM
  { clLnDisplay(8 - line) ; }         // erase each in turn, home cursor to line
  onDisplay() ;                       // turn on OLED panel
}

void clLnDisplay(byte yline)
{ // Clear single line 0-7 on display, cursor to line start
  byte ram ;
  cursorXY(0, yline)    ;            // Cursor Home
  for (ram=sWidth ; ram>0 ; ram--)   // sWidth DDRAM addresses in each line
  { sendData(0); }                   // clear RAM
  cursorXY(0, yline) ;               // Cursor Home
}

void rptPix(byte pix, byte blank, byte percent)
{ // repeat pixel across Display for percent - show bar graph
  byte count ;
  int width ;
  width = (sWidth*percent)/100 ; // screen width x fill level
  for (count=0; count<sWidth; count++)  // loop for width
  {
   if ((count <= width)||(count > (sWidth - 2)))
   {  sendData(pix) ; }  // send 'pix' glyph to display
   else
   {  sendData(blank) ;} // send 'blank' glyph to display
  }
}

void barDisplay(byte barG)
{// water bargraph on line 7  
  cursorXY(0, 7) ;
  rptPix(0b11111111, 0b10000001, barG);  // Draw line, then blanks
}

// Display FONTS:
// 5x8 (1-line, full 7-bit ASCII from space)
// 34x48 (6 lines, numerals only, ASCII 48 - 57) Calibri Bold 54

// Font5 functions (5x8)

void charFont5(byte character) // MODIFY IF FONT RANGE CHANGED
{ // Prepare to write byte to Font5, via sendGlyphs
  if (character == ' ')  // munge ' ' to empty ';' character
   { character = ';' ; }
  if ((character<'.')||(character>'Z')) // reduced font5 table, CAPS only
   { return ; }    // Exit function if character out of ASCII range
  //Index to row of the character in font5 table, then send glyphs
  sendGlyphs(font5[character - '.'], sizeof font5[0]);
}

void strFont5(byte s[])
{ // Write Variable or Contatant string to Font5
  while(*s) {charFont5(*s++);} // Points to character, or terminator
}

void strFont5XY(byte s[], byte x, byte y)
{ // Write (V or C) string to Font5 @ XY
  cursorXY(x,y);
  strFont5(s);
}

void byteFont5XY(byte num, byte x, byte y)
{ // display byte value at XY
  char txt4[4];
  ByteToStr(num, txt4);
  strFont5XY(txt4, x, y);
}

void intFont5XY(int num, byte x, byte y)
{ // display int value at XY
  char txt7[7];
  IntToStr(num, txt7);
  strFont5XY(txt7, x, y);
}

void byteFont5(byte num)
{ // display byte value at cursor
  char txt4[4];
  ByteToStr(num, txt4);
  strFont5(txt4);
}

void digitFont5(byte digit) // send single digit to Display
{
  charFont5(digit + '0');
}

void digitFont5XY(byte digit, byte x, byte y) // send single digit to Display @ x, y
{
  cursorXY(x,y);
  digitFont5(digit) ;
}

// End of Font5 functions

// Font6R functions (34x48) numerals only

void charfont6Ra(byte character)
{ // Prepare top 22 bytes for Display, via sendGlyphs (numerals, space and '.' only)
  //if (character==' ') {character='/';} // Munge ' ' into '/' for blank glyph
  if ((character<'0')||(character>'9'))
  { return ; }    // Exit function if character out of numeric range
  //Index to row of the character in font6R table, then send glyphs
  sendGlyphs(font6Ra[character - '0'], sizeof font6Ra[0]);
}

void charfont6Rb(byte character)
{ // Prepare middle1 22 bytes for Display, via sendGlyphs
  // if (character==' ') {character='/';} // Munge ' ' into '/' for blank glyph
  if ((character<'0')||(character>'9'))
  { return ; }    // Exit function if character out of numeric range
  //Index to row of the character in font6R table, then send glyphs
  sendGlyphs(font6Rb[character - '0'], sizeof font6Rb[0]);
}

void charfont6Rc(byte character)
{ // Prepare middle2 22 bytes for Display, via sendGlyphs
  // if (character==' ') {character='/';} // Munge ' ' into '/' for blank glyph
  if ((character<'0')||(character>'9'))
  { return ; }     // Exit function if character out of numeric range
  //Index to row of the character in font6R table, then send glyphs
  sendGlyphs(font6Rc[character - '0'], sizeof font6Rc[0]);
}

void charfont6Rd(byte character)
{ // Prepare lower 22 bytes for Display, via sendGlyphs
  // if (character==' ') {character='/';} // Munge ' ' into '/' for blank glyph
  if ((character<'0')||(character>'9'))
  { return ; }   // Exit function if character out of numeric range
  //Index to row of the character in font6R table, then send glyphs
  sendGlyphs(font6Rd[character - '0'], sizeof font6Rd[0]);
}

void charfont6Re(byte character)
{ // Prepare lower 22 bytes for Display, via sendGlyphs
  // if (character==' ') {character='/';} // Munge ' ' into '/' for blank glyph
  if ((character<'0')||(character>'9'))
  { return ; }   // Exit function if character out of numeric range
  //Index to row of the character in font6R table, then send glyphs
  sendGlyphs(font6Re[character - '0'], sizeof font6Re[0]);
}

void charfont6Rf(byte character)
{ // Prepare lower 22 bytes for Display, via sendGlyphs
  // if (character==' ') {character='/';} // Munge ' ' into '/' for blank glyph
  if ((character<'0')||(character>'9'))
  { return ; }   // Exit function if character out of numeric range
  //Index to row of the character in font6R table, then send glyphs
  sendGlyphs(font6Rf[character - '0'], sizeof font6Rf[0]);
}

void strfont6Ra(byte s[])
{ // Write (V or C) string to font6Ra
  while(*s) {charfont6Ra(*s++);} // Points to character, or terminator
}

void strfont6Rb(byte s[])
{ // Write (V or C) string to font6Rb
  while(*s) {charfont6Rb(*s++);} // Points to character, or terminator
}

void strfont6Rc(byte s[])
{ // Write (V or C) string to font6Rc
  while(*s) {charfont6Rc(*s++);} // Points to character, or terminator
}

void strfont6Rd(byte s[])
{ // Write (V or C) string to font6Rd
  while(*s) {charfont6Rd(*s++);} // Points to character, or terminator
}

void strfont6Re(byte s[])
{ // Write (V or C) string to font6Rd
  while(*s) {charfont6Re(*s++);} // Points to character, or terminator
}

void strfont6Rf(byte s[])
{ // Write (V or C) string to font6Rd
  while(*s) {charfont6Rf(*s++);} // Points to character, or terminator
}

void strfont6RXY(byte s[], byte x, byte y)
{ // Write (V or C) string to font6R @ XY
  cursorXY(x, y) ;
    strfont6Ra(s) ;
  cursorXY(x, y+1) ;
    strfont6Rb(s) ;
  cursorXY(x, y+2) ;
    strfont6Rc(s) ;
  cursorXY(x, y+3) ;
    strfont6Rd(s) ;
  cursorXY(x, y+4) ;
    strfont6Re(s) ;
  cursorXY(x, y+5) ;
    strfont6Rf(s) ;
}
// End of font6R functions

// Misc init command repository

   // sendCmd(0x0F & sXOffset) ; // set low column address to any offset              [#1]
//  sendCmd(0x10) ;            // set high column address to 0                      [#2]
  // sendCmd(0xD9) ;   // set pre-charge period [#16]
  // sendCmd(0x25) ; // default is 22
  // sendCmd(0xDB) ;   // set Vcom Deselect Level [#18]
  // sendCmd(0x20) ; // set Vcom Deselect Level [POR = 35]
  // sendCmd(0xF1) ;        // Set Pre-Charge as 15 Clocks & Discharge as 1 Clock
  // clsDisplay ;
  
  
  //  sendCmd(0x32) ;    // set Charge Pump voltage 0x32 = 8V                              [#3]
//  sendCmd(0xAD) ;    // DC-DC Control Mode                                            [#10]
//    sendCmd(0x8B) ;  // DC-DC Set ON

// sendCmd(0x8D) ;    // set SSD1306 Charge Pump enable/disable [see supplement]
//   sendCmd(0x14) ;  // enable charge pump on SSD1306


  /*
  sendCmd(0xA8) ;    // set multiplex ratio (1 to 64)                             [#9]
    sendCmd(0x3F) ;  // [POR = 3F] 1/64 duty
  sendCmd(0xD3) ;    // set display offset Shift Mapping RAM Counter (0x00~0x3F) [#14]
    sendCmd(0x00) ;  // [POR = 0] not offset
  sendCmd(0x40) ;    // set start line to 0                                       [#4]
  sendCmd(0xA0) ;    // set SEG/Column left re-map [POR = A0]                     [#6]
  sendCmd(0xC0) ;    // set COM scan direction 0-n [POR = C0]                     [#13]
  sendCmd(0xDA) ;    // set com pins hardware configuration                       [#17]
    sendCmd(0x12) ;  // [POR = 12]
  sendCmd(0x81) ;    // set contrast control register                             [#5]
    sendCmd(0xF0) ;  //  80 [POR]

  sendCmd(0xA6) ;    // set normal display A6 [POR] (reverse = A7)                [#8]
  sendCmd(0xD5) ;    // set display clock divide ratio/oscillator frequency       [#15]
    sendCmd(0x50) ;  // set divide ratio 0, Set Clock*/