// DIGITAL WATER GAUGE using SSD1106 OLED Driver
// Version 4.0 by Nigel Mercier PIC16F1847

// mikroElektronica Libraries used: ADC, CONVERSIONS, EEPROM, C_String

#include <built_in.h>
#include "config.h"
#include "SH1106.h"
#include "waterg.h"
#include "functions.h"

// Global Constants (tank tables)
// const byte mult = 5 ; //  multiplier for values in LitresAft[] and LitresFwd[]

// Sensor... is expected sensor readings, Litres... is litres/mult
const byte SizeAft = 16 ;   // number of steps AFT sensor, also array size
const byte SensorAft[SizeAft] = {236, 220, 201, 177, 171, 164, 156, 149, 140, 131, 117, 101, 83, 62, 39, 11} ;
const byte LitresAft[SizeAft] = {0,   1,   2,   3,   5,   7,   9,   11,  14,  17,  21,  26,  31, 37, 44, 49} ;

const byte SizeFwd = 20 ;   // number of steps FWD sensor, also array size
const byte SensorFwd[SizeFwd] = {235, 224, 211, 195, 177, 171, 164, 156, 149, 140, 132, 124, 114, 104, 93, 82, 66, 49, 31, 10} ;
const byte LitresFwd[SizeFwd] = {0,   0,   0,   0,   1,   2,   4,   6,   9,   12,  15,  19,  22,  26,  30, 34, 39, 44, 49, 57} ;

// Global Constants (text)
char ErrTab[2][16] = {{"WIRE BREAK"}, {"WIRE SHORT"}} ;

// Global variables (temp)
char txt4[4] , txt7[7];   // temporary for conversion strings and
byte i, j;                // misc loops
// Global variables
int rawADC ;          // raw ADC value for monitor mode
byte adc8 ;           // 8-bit value from ADC
byte index ;          // index of closest value in SensorAft or SensorFwd
byte litVal ;         // litre value of index in LitresAft[] or LitresFwd[]
byte barG ;           // fraction of tank level over last array entry
byte mode ;           // copy of swMode at entry

// Global variables set in ISR
byte swMode = 0 ;           // mode of switch 0, 1 or 2
byte timeSec = 0 ;          // 6 second counter
byte disFlag = 1 ;          // display flag set every 1 sec
byte errFlag = 0 ;          // flag error for out of range ADC reading


//-------------- Main Program -----------

void main( void )
{
  configReg() ;         // configure SFRs
  configInt() ;         // configure interrupts
  initDisplay() ;       // reset and initialise display

  strFont5XY("WATER GAUGE", 30, 1) ;
  strFont5XY("S/Y DOLCETTO", 27, 2) ;
  strFont5XY("SWITCH: MONITOR MODE", 3, 7) ;
  
  beep() ;            // beep to indicate power on
  delayX25ms(80) ;    // allow 2 seconds for switch press for monitor mode
  beep() ;            // beep to indicate time up
  
  clsDisplay() ;
  mode = ((PORTB >> 6) ^ 0b00000011) ; // shift >> 6 to b1-b0, toggle bits
  // swMode set to 0 (released or no switch), 1 (b7 SWf) or 2 (b6 SWa)
  if (mode)               // has key been pressed during boot screen
  { goto monitorMode ; }  // jump to monitor mode, power off to exit

/**************************************************
// main loop - with interrupt updates every second
***************************************************/

  while (1)
  { // infinite loop driven by interrupts
    if (swMode) // swMode set to 0 (no switch), 1 (b7 SWf) or 2 (b6 SWa)
    {
      mode = swMode ; // changed by ISR so save for tests
      LED = 0 ;   // DEBUG LED ON
      beep() ;        // beep to indicate switch pressed
      clsDisplay();   // clear screen and turn on

      if (mode == 1) // FWD tank switch pressed
      {  // use: ADC(1), FETf, findNearF, SensorFwd[], LitresFwd[]
       modeF() ;
      }
      else if (mode == 2) // AFT tank switch pressed
      { // use: ADC(0), FETa, findNearA, SensorAft[], LitresAft[]
       modeA() ;
      }

      // got values, check for errors
      timeSec = 0 ;              // start timer
      if (errFlag)               // errFlag: 1 high ADC (O/C), 0 low ADC (S/C)
      {
      strFont5XY(ErrTab[errFlag - 1], 13, 0) ;
      errFlag = 0 ;
      }
      else
      { // display and wait
        rawADC = ADC_Read(LightSens) ; // 0 (dark) to 1023 (bright)
        setContrast(rawADC / 4) ;      // contrast range 0 - 255
        barDisplay(barG) ;             // bargraph on line 7
        displayLarge(litVal*5) ;       // display litres in large 6 line font
        // intFont5XY(rawADC, 0, 6) ;  // DEBUG display light sensor value
      }
      
      swMode = 0 ;
      while( (timeSec < 6) && (!swMode) )
      {
      // byteFont5XY(timeSec, 0, 6) ;   // DEBUG display count down value
      }
      LED = 1 ;    // DEBUG LED OFF

      clsDisplay();       // clear screen
      offDisplay() ;      // turn off screen
      // asm sleep ;   // check if worth it
    }       
  }

  monitorMode:
  { //  monitor mode, key pressed on boot screen, power off to exit
    int lastADC = 0 ;  // to save last ADC reading and note change
    int delta ;        // difference last and new ADC
    LED = 0 ;          // DEBUG LED ON
    while (1)
    {   
      if (mode == 1) // FWD tank switch was pressed at boot
      { modeF() ; }  // use: ADC(1), FETf, findNearF, SensorFwd[], LitresFwd[]

      if (mode == 2) // AFT tank switch was pressed at boot
      { modeA() ; }  // use: ADC(0), FETa, findNearA, SensorAft[], LitresAft[]

      intFont5XY(rawADC, 38, 0) ;    // display ADC reading
      strFont5XY("ADC = ", 14, 0) ;  // display legend
      delta = absVal(rawADC - lastADC) ;
      if (delta > 5)                 // ADC reading changed, new sensor step
      {
        strFont5XY("LOG L", 14, 0) ;  // advise to log new litre value
        for (i = 0; i < 6; i++)
        { beep() ; }                 // long beep to indicate change
      }
      lastADC = rawADC ;             // ready for next
      delayX25ms(40) ;               // ... wait one second
    }
  }

}    // end of main()